@@ -1,12 +1,12 @@
 package Perinci::CmdLine::Lite;
 
-our $DATE = '2017-07-12'; # DATE
-our $VERSION = '1.75_001'; # VERSION
+our $DATE = '2015-03-26'; # DATE
+our $VERSION = '1.01'; # VERSION
 
 use 5.010001;
 # use strict; # already enabled by Mo
 # use warnings; # already enabled by Mo
-use Log::ger;
+use Log::Any::IfLOG '$log';
 
 use List::Util qw(first);
 use Mo qw(build default);
@@ -21,12 +21,39 @@ has default_prompt_template => (
     is=>'rw',
     default => 'Enter %s: ',
 );
-has validate_args => (
+has log => (
     is=>'rw',
-    default => 1,
+    default => sub {
+        if (defined $ENV{LOG}) {
+            return $ENV{LOG};
+        } elsif ($ENV{LOG_LEVEL} && $ENV{LOG_LEVEL} =~ /^(off|none)$/) {
+            return 0;
+        } elsif ($ENV{LOG_LEVEL} || $ENV{TRACE} || $ENV{DEBUG} ||
+                     $ENV{VERBOSE} || $ENV{QUIET}) {
+            return 0;
+        }
+        0;
+    },
+);
+has log_level => (
+    is=>'rw',
+    default => sub {
+        if ($ENV{LOG_LEVEL}) {
+            return $ENV{LOG_LEVEL};
+        } elsif ($ENV{TRACE}) {
+            return 'trace';
+        } elsif ($ENV{DEBUG}) {
+            return 'debug';
+        } elsif ($ENV{VERBOSE}) {
+            return 'info';
+        } elsif ($ENV{QUIET}) {
+            return 'error';
+        }
+        'warning';
+    },
 );
 
-my $formats = [qw/text text-simple text-pretty json json-pretty csv html html+datatables/];
+my $formats = [qw/text text-simple text-pretty json json-pretty/];
 
 sub BUILD {
     my ($self, $args) = @_;
@@ -61,15 +88,12 @@ sub BUILD {
 
         $copts->{version}   = { $_t->('version'), };
         $copts->{help}      = { $_t->('help'), };
-
-        unless ($self->skip_format) {
-            $copts->{format}    = {
-                $_t->('format'),
-                schema => ['str*' => in => $formats],
-            };
-            $copts->{json}      = { $_t->('json'), };
-            $copts->{naked_res} = { $_t->('naked_res'), };
-        }
+        $copts->{format}    = {
+            $_t->('format'),
+            schema => ['str*' => in => $formats],
+        };
+        $copts->{json}      = { $_t->('json'), };
+        $copts->{naked_res} = { $_t->('naked_res'), };
         if ($self->subcommands) {
             $copts->{subcommands} = { $_t->('subcommands'), };
         }
@@ -81,9 +105,6 @@ sub BUILD {
             $copts->{no_config}      = { $_t->('no_config') };
             $copts->{config_profile} = { $_t->('config_profile') };
         }
-        if ($self->read_env) {
-            $copts->{no_env} = { $_t->('no_env') };
-        }
         if ($self->log) {
             $copts->{log_level} = { $_t->('log_level'), };
             $copts->{trace}     = { $_t->('trace'), };
@@ -114,12 +135,13 @@ sub _unsetup_progress_output {
     my $self = shift;
 
     return unless $setup_progress;
-    no warnings 'once';
     my $out = $Progress::Any::outputs{''}[0];
     $out->cleanup if $out->can("cleanup");
     $setup_progress = 0;
 }
 
+sub hook_before_run {}
+
 sub hook_after_parse_argv {
     my ($self, $r) = @_;
 
@@ -137,132 +159,180 @@ sub hook_after_parse_argv {
             $args->{$_} = $as->{schema}[1]{default};
         }
     }
+
+    # set up log adapter
+    if ($self->log) {
+        require Log::Any::Adapter;
+        Log::Any::Adapter->set(
+            'ScreenColoredLevel',
+            min_level => $r->{log_level} // $self->log_level,
+            formatter => sub { $self->program_name . ": $_[1]" },
+        );
+    }
 }
 
-# BEGIN_BLOCK: equal2
-sub equal2 {
-    state $require = do { require Scalar::Util };
+sub __json {
+    state $json = do {
+        require JSON;
+        JSON->new->canonical(1)->allow_nonref;
+    };
+    $json;
+}
 
-    # for lack of a better name, currently i name this 'equal2'
-    my ($val1, $val2) = @_;
+sub __gen_table {
+    my ($data, $header_row, $resmeta) = @_;
 
-    # here are the rules:
-    # - if both are undef, 1
-    # - undef equals nothing else
-    # - if both are ref, equal if their refaddr() are equal
-    # - if only one is ref, 0
-    # - if none is ref, compare using eq
+    $resmeta //= {};
 
-    if (defined $val1) {
-        return 0 unless defined $val2;
-        if (ref $val1) {
-            return 0 unless ref $val2;
-            return Scalar::Util::refaddr($val1) eq Scalar::Util::refaddr($val2);
-        } else {
-            return 0 if ref $val2;
-            return $val1 eq $val2;
-        }
+    my @columns;
+    if ($header_row) {
+        @columns = @{$data->[0]};
     } else {
-        return 0 if defined $val2;
-        return 1;
+        @columns = map {"col$_"} 0..@{$data->[0]}-1;
     }
-}
-# END_BLOCK: equal2
 
-sub hook_before_action {
-    my ($self, $r) = @_;
+    my $column_orders; # e.g. [col2, col1, col3, ...]
+  SET_COLUMN_ORDERS: {
 
-    # validate arguments using schema from metadata
-  VALIDATE_ARGS:
-    {
-        last unless $self->validate_args;
-
-        # unless we're feeding the arguments to function, don't bother
-        # validating arguments
-        last unless $r->{action} eq 'call';
-
-        my $meta = $r->{meta};
-
-        # function is probably already wrapped
-        last if $meta->{'x.perinci.sub.wrapper.logs'} &&
-            (grep { $_->{validate_args} }
-             @{ $meta->{'x.perinci.sub.wrapper.logs'} });
-
-        require Data::Sah;
-
-        # to be cheap, we simply use "$ref" as key as cache key. to be proper,
-        # it should be hash of serialized content.
-        my %validators; # key = "$schema"
-
-        for my $arg (sort keys %{ $meta->{args} // {} }) {
-            next unless exists($r->{args}{$arg});
-
-            # we don't support validation of input stream because this must be
-            # done after each 'get item' (but periswrap does)
-            next if $meta->{args}{$arg}{stream};
-
-            my $schema = $meta->{args}{$arg}{schema};
-            next unless $schema;
-            unless ($validators{"$schema"}) {
-                my $v = Data::Sah::gen_validator($schema, {
-                    return_type => 'str+val',
-                    schema_is_normalized => 1,
-                });
-                $validators{"$schema"} = $v;
+        # find column orders from 'table_column_orders' in result metadata (or
+        # from env)
+        my $tcos;
+        if ($ENV{FORMAT_PRETTY_TABLE_COLUMN_ORDERS}) {
+            $tcos = __json->decode($ENV{FORMAT_PRETTY_TABLE_COLUMN_ORDERS});
+        } elsif (my $rfos = ($resmeta->{'cmdline.format_options'} //
+                                 $resmeta->{format_options})) {
+            my $rfo = $rfos->{'text-pretty'} // $rfos->{text} // $rfos->{any};
+            if ($rfo) {
+                $tcos = $rfo->{table_column_orders};
             }
-            my $res = $validators{"$schema"}->($r->{args}{$arg});
-            if ($res->[0]) {
-                die [400, "Argument '$arg' fails validation: $res->[0]"];
-            }
-            my $val0 = $r->{args}{$arg};
-            my $coerced_val = $res->[1];
-            $r->{args}{$arg} = $coerced_val;
-            $r->{args}{"-orig_$arg"} = $val0 unless equal2($val0, $coerced_val);
         }
-
-        if ($meta->{args_rels}) {
-            my $schema = [hash => $meta->{args_rels}];
-            my $sah = Data::Sah->new;
-            my $hc  = $sah->get_compiler("human");
-            my $cd  = $hc->init_cd;
-            $cd->{args}{lang} //= $cd->{default_lang};
-            my $v = Data::Sah::gen_validator($schema, {
-                return_type => 'str',
-                human_hash_values => {
-                    field  => $hc->_xlt($cd, "argument"),
-                    fields => $hc->_xlt($cd, "arguments"),
-                },
-            });
-            my $res = $v->($r->{args});
-            if ($res) {
-                die [400, $res];
+        if ($tcos) {
+            # find an entry in tcos that @columns contains all the columns of
+          COLS:
+            for my $cols (@$tcos) {
+                for my $col (@$cols) {
+                    next COLS unless first {$_ eq $col} @columns;
+                }
+                $column_orders = $cols;
+                last SET_COLUMN_ORDERS;
             }
         }
 
+        # find column orders from table spec
+        $column_orders = $resmeta->{'table.fields'};
     }
+
+    # reorder each row according to requested column order
+    if ($column_orders) {
+        require List::MoreUtils;
+
+        # 0->2, 1->0, ... (map column position from unordered to ordered)
+        my @map0 = sort {
+            my $idx_a = List::MoreUtils::firstidx(sub {$_ eq $a->[1]},
+                                                  @$column_orders) // 9999;
+            my $idx_b = List::MoreUtils::firstidx(sub {$_ eq $b->[1]},
+                                                  @$column_orders) // 9999;
+            $idx_a <=> $idx_b || $a->[1] cmp $b->[1];
+        } map {[$_, $columns[$_]]} 0..@columns-1;
+        #use DD; dd \@map0;
+        my @map;
+        for (0..@map0-1) {
+            $map[$_] = $map0[$_][0];
+        }
+        #use DD; dd \@map;
+        my $newdata = [];
+        for my $row (@$data) {
+            my @newrow;
+            for (0..@map-1) { $newrow[$_] = $row->[$map[$_]] }
+            push @$newdata, \@newrow;
+        }
+        $data = $newdata;
+    }
+
+    Text::Table::Tiny::table(rows=>$data, header_row=>$header_row) . "\n";
 }
 
 sub hook_format_result {
-    require Perinci::Result::Format::Lite;
     my ($self, $r) = @_;
 
-    my $fmt = $r->{format} // 'text';
+    my $res    = $r->{res};
+    my $format = $r->{format} // 'text';
+    my $meta   = $r->{meta};
 
-    if ($fmt eq 'html+datatables') {
-        $fmt = 'text-pretty';
-        $ENV{VIEW_RESULT} //= 1;
-        $ENV{FORMAT_PRETTY_TABLE_BACKEND} //= 'Text::Table::HTML::DataTables';
+    if ($format =~ /\Atext(-simple|-pretty)?\z/) {
+        my $is_pretty = $format eq 'text-pretty' ? 1 :
+            $format eq 'text-simple' ? 0 : (-t STDOUT);
+        no warnings 'uninitialized';
+        if ($res->[0] !~ /^(2|304)/) {
+            my $fres = "ERROR $res->[0]: $res->[1]";
+            if (my $prev = $res->[3]{prev}) {
+                $fres .= " ($prev->[0]: $prev->[1])";
+            }
+            return "$fres\n";
+        } elsif ($res->[3] && $res->[3]{"x.hint.result_binary"}) {
+            return $res->[2];
+        } else {
+            require Data::Check::Structure;
+            my $data = $res->[2];
+            my $max = 5;
+            if (!ref($data)) {
+                $data //= "";
+                $data .= "\n" unless !length($data) || $data =~ /\n\z/;
+                return $data;
+            } elsif (ref($data) eq 'ARRAY' && !@$data) {
+                return "";
+            } elsif (Data::Check::Structure::is_aos($data, {max=>$max})) {
+                return join("", map {"$_\n"} @$data);
+            } elsif (Data::Check::Structure::is_aoaos($data, {max=>$max})) {
+                if ($is_pretty) {
+                    require Text::Table::Tiny;
+                    return __gen_table($data, 0, $res->[3]);
+                } else {
+                    return join("", map {join("\t", @$_)."\n"} @$data);
+                }
+            } elsif (Data::Check::Structure::is_hos($data, {max=>$max})) {
+                if ($is_pretty) {
+                    require Text::Table::Tiny;
+                    $data = [map {[$_, $data->{$_}]} sort keys %$data];
+                    unshift @$data, ["key", "value"];
+                    return __gen_table($data, 1, $res->[3]);
+                } else {
+                    return join("", map {"$_\t$data->{$_}\n"} sort keys %$data);
+                }
+            } elsif (Data::Check::Structure::is_aohos($data, {max=>$max})) {
+                # collect all mentioned fields
+                my %fieldnames;
+                for my $row (@$data) {
+                    $fieldnames{$_}++ for keys %$row;
+                }
+                my @fieldnames = sort keys %fieldnames;
+                my $newdata = [];
+                for my $row (@$data) {
+                    push @$newdata, [map {$row->{$_}} @fieldnames];
+                }
+                if ($is_pretty) {
+                    unshift @$newdata, \@fieldnames;
+                    require Text::Table::Tiny;
+                    return __gen_table($newdata, 1, $res->[3]);
+                } else {
+                    return join("", map {join("\t", @$_)."\n"} @$newdata);
+                }
+            } else {
+                $format = 'json-pretty';
+            }
+        }
     }
 
-    my $fres = Perinci::Result::Format::Lite::format(
-        $r->{res}, $fmt, $r->{naked_res}, $self->{use_cleanser});
+    $res = $res->[2] if $r->{naked_res};
 
-    # ux: prefix error message with program name
-    if ($fmt =~ /text/ && $r->{res}[0] =~ /\A[45]/ && defined($r->{res}[1])) {
-        $fres = $self->program_name . ": $fres";
+    warn "Unknown format '$format', fallback to json-pretty"
+        unless $format =~ /\Ajson(-pretty)?\z/;
+    $self->cleanser->clean_in_place($res);
+    if ($format eq 'json') {
+        return __json->encode($res) . "\n";
+    } else {
+        return __json->canonical(1)->pretty->encode($res);
     }
-
-    $fres;
 }
 
 sub hook_format_row {
@@ -277,27 +347,6 @@ sub hook_format_row {
 
 sub hook_display_result {
     my ($self, $r) = @_;
-
-    my $res  = $r->{res};
-    my $resmeta = $res->[3] // {};
-
-    my $handle = $r->{output_handle};
-
-    # set utf8 flag
-    my $utf8;
-    {
-        last if defined($utf8 = $ENV{UTF8});
-        if ($resmeta->{'x.hint.result_binary'}) {
-            # XXX only when format is text?
-            $utf8 = 0; last;
-        }
-        if ($r->{subcommand_data}) {
-            last if defined($utf8 = $r->{subcommand_data}{use_utf8});
-        }
-        $utf8 = $self->use_utf8;
-    }
-    binmode($handle, ":utf8") if $utf8;
-
     $self->display_result($r);
 }
 
@@ -310,26 +359,15 @@ sub hook_after_get_meta {
     my ($self, $r) = @_;
 
     require Perinci::Object;
-    my $metao = Perinci::Object::risub($r->{meta});
-    if ($metao->can_dry_run) {
-        my $default_dry_run = $metao->default_dry_run // $self->default_dry_run;
-        $r->{dry_run} = 1 if $default_dry_run;
-        $r->{dry_run} = ($ENV{DRY_RUN} ? 1:0) if defined $ENV{DRY_RUN};
+    if (Perinci::Object::risub($r->{meta})->can_dry_run) {
         $self->common_opts->{dry_run} = {
-            getopt  => $default_dry_run ? 'dry-run!' : 'dry-run',
-            summary => $default_dry_run ?
-                "Disable simulation mode (also via DRY_RUN=0)" :
-                "Run in simulation mode (also via DRY_RUN=1)",
+            getopt  => 'dry-run',
+            summary => "Run in simulation mode (also via DRY_RUN=1)",
             handler => sub {
                 my ($go, $val, $r) = @_;
-                if ($val) {
-                    log_debug("[pericmd] Dry-run mode is activated");
-                    $r->{dry_run} = 1;
-                    #$ENV{VERBOSE} = 1;
-                } else {
-                    log_debug("[pericmd] Dry-run mode is deactivated");
-                    $r->{dry_run} = 0;
-                }
+                $log->debugf("[pericmd] Dry-run mode is activated");
+                $r->{dry_run} = 1;
+                #$ENV{VERBOSE} = 1;
             },
         };
     }
@@ -337,7 +375,7 @@ sub hook_after_get_meta {
     # check deps property. XXX this should be done only when we don't wrap
     # subroutine, because Perinci::Sub::Wrapper already checks the deps
     # property.
-    if ($r->{meta}{deps} && !$r->{in_dump} && !$r->{in_completion}) {
+    if ($r->{meta}{deps}) {
         require Perinci::Sub::DepChecker;
         my $res = Perinci::Sub::DepChecker::check_deps($r->{meta}{deps});
         if ($res) {
@@ -346,7 +384,7 @@ sub hook_after_get_meta {
     }
 }
 
-sub action_subcommands {
+sub run_subcommands {
     my ($self, $r) = @_;
 
     if (!$self->subcommands) {
@@ -365,43 +403,25 @@ sub action_subcommands {
       )];
 }
 
-sub action_version {
-    no strict 'refs';
-
+sub run_version {
     my ($self, $r) = @_;
 
-    my @text;
+    my $meta = $r->{meta} = $self->get_meta($r, $self->url);
 
-    {
-        my $meta = $r->{meta} = $self->get_meta($r, $self->url);
-        push @text, $self->get_program_and_subcommand_name($r),
-            " version ", ($meta->{entity_v} // "?"),
-            ($meta->{entity_date} ? " ($meta->{entity_date})" : ''),
-            "\n";
-        for my $mod (@{ $meta->{'x.dynamic_generator_modules'} // [] }) {
-            push @text, "  $mod version ", ${"$mod\::VERSION"},
-                (${"$mod\::DATE"} ? " (".${"$mod\::DATE"}.")" : ""),
-                    "\n";
-        }
-    }
-
-    for my $url (@{ $self->extra_urls_for_version // [] }) {
-        my $meta = $self->get_meta($r, $url);
-        push @text, "  $url version ", ($meta->{entity_v} // "?"),
-            ($meta->{entity_date} ? " ($meta->{entity_date})" : ''),
-            "\n";
-    }
-
-    push @text, "  ", __PACKAGE__,
-        " version ", ($Perinci::CmdLine::Lite::VERSION // "?"),
-        ($Perinci::CmdLine::Lite::DATE ?
-         " ($Perinci::CmdLine::Lite::DATE)":''),
-        "\n";
-
-    [200, "OK", join("", @text)];
+    [200, "OK",
+     join("",
+          $self->get_program_and_subcommand_name($r),
+          " version ", ($meta->{entity_v} // "?"),
+          ($meta->{entity_date} ? " ($meta->{entity_date})" : ''),
+          "\n",
+          "  ", __PACKAGE__,
+          " version ", ($Perinci::CmdLine::Lite::VERSION // "?"),
+          ($Perinci::CmdLine::Lite::DATE ?
+               " ($Perinci::CmdLine::Lite::DATE)":''),
+      )];
 }
 
-sub action_help {
+sub run_help {
     require Perinci::CmdLine::Help;
 
     my ($self, $r) = @_;
@@ -410,17 +430,15 @@ sub action_help {
     my $scn    = $r->{subcommand_name};
     my $scd    = $r->{subcommand_data};
 
-    my $meta = $self->get_meta($r, $scd->{url} // $self->{url});
-
     # XXX use 'delete local' when we bump minimal perl to 5.12
     my $common_opts = { %{$self->common_opts} };
-
     # hide usage '--subcommands' if we have subcommands but user has specified a
     # subcommand to use
-    my $has_sc_no_sc = $self->subcommands &&
-        !length($r->{subcommand_name} // '');
+    my $has_sc_no_sc = $self->subcommands && !length($r->{subcommand_name});
     delete $common_opts->{subcommands} if $self->subcommands && !$has_sc_no_sc;
 
+    my $meta = $self->get_meta($r, $scd->{url} // $self->{url});
+
     my $res = Perinci::CmdLine::Help::gen_help(
         program_name => $self->get_program_and_subcommand_name($r),
         program_summary => ($scd ? $scd->{summary}:undef ) // $meta->{summary},
@@ -436,12 +454,12 @@ sub action_help {
     $res;
 }
 
-sub action_call {
+sub run_call {
     my ($self, $r) = @_;
 
     my %extra;
     if ($r->{send_argv}) {
-        log_trace("[pericmd] Sending argv to server: %s", $extra{argv});
+        $log->tracef("[pericmd] Sending argv to server: %s", $extra{argv});
         $extra{argv} = $r->{orig_argv};
     } else {
         my %extra_args;
@@ -454,9 +472,7 @@ sub action_call {
     my $url = $r->{subcommand_data}{url};
 
     # currently we don't log args because it's potentially large
-    log_trace("[pericmd] Riap request: action=call, url=%s", $url);
-
-    #log_trace("TMP: extra=%s", \%extra);
+    $log->tracef("[pericmd] Riap request: action=call, url=%s", $url);
 
     # setup output progress indicator
     if ($r->{meta}{features}{progress}) {
@@ -468,7 +484,7 @@ sub action_call {
 }
 
 1;
-# ABSTRACT: A Rinci/Riap-based command-line application framework
+# ABSTRACT: A lightweight Rinci/Riap-based command-line application framework
 
 __END__
 
@@ -478,11 +494,11 @@ __END__
 
 =head1 NAME
 
-Perinci::CmdLine::Lite - A Rinci/Riap-based command-line application framework
+Perinci::CmdLine::Lite - A lightweight Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-This document describes version 1.75_001 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2017-07-12.
+This document describes version 1.01 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2015-03-26.
 
 =head1 SYNOPSIS
 
@@ -586,12 +602,11 @@ Automatic tab completion support:
  % complete -C gen-random-num gen-random-num
  % gen-random-num --mi<tab>
 
-See L<Perinci::CmdLine::Manual> for details on other available features
-(subcommands, automatic formatting of data structures, automatic schema
-validation, dry-run mode, automatic POD generation, remote function support,
-automatic CLI generation, automatic --version, automatic HTTP API,
-undo/transactions, configurable output format, logging, progress bar,
-colors/Unicode, and more).
+See the manual for details on other available features (subcommands, automatic
+formatting of data structures, automatic schema validation, dry-run mode,
+automatic POD generation, remote function support, automatic CLI generation,
+automatic --version, automatic HTTP API, undo/transactions, configurable output
+format, logging, progress bar, colors/Unicode, and more).
 
 =head1 DESCRIPTION
 
@@ -603,14 +618,11 @@ See L<Perinci::CmdLine::Manual> for more details.
 There is also a blog post series on Perinci::CmdLine tutorial:
 L<https://perlancar.wordpress.com/category/pericmd-tut/>
 
-Perinci::CmdLine::Lite is the default backend implementation. Another
-implementation is the heavier L<Perinci::CmdLine::Classic> which has a couple of
-more features not yet incorporated into ::Lite, e.g. transactions.
+Perinci::CmdLine::Lite is the default backend implementation. You normally
+should use L<Perinci::CmdLine::Any> instead to be able to switch backend on the
+fly.
 
-You normally should use L<Perinci::CmdLine::Any> instead to be able to switch
-backend on the fly.
-
-=for Pod::Coverage ^(BUILD|get_meta|hook_.+|action_.+)$
+=for Pod::Coverage ^(BUILD|get_meta|hook_.+|run_.+)$
 
 =head1 REQUEST KEYS
 
@@ -620,20 +632,24 @@ All those supported by L<Perinci::CmdLine::Base>, plus:
 
 =back
 
-=head1 RESULT METADATA
-
-All those supported by L<Perinci::CmdLine::Base>, plus:
-
-=head2 x.hint.result_binary => bool
-
-If set to true, then when formatting to C<text> formats, this class won't print
-any newline to keep the data being printed unmodified.
-
 =head1 ATTRIBUTES
 
 All the attributes of L<Perinci::CmdLine::Base>, plus:
 
-=head2 validate_args => bool (default: 1)
+=head2 log => bool (default: 0, or from env)
+
+Whether to enable logging. This currently means setting up L<Log::Any::Adapter>
+to display logging (set in C<hook_after_parse_argv>, so tab completion skips
+this step). To produce log, you use L<Log::Any> in your code.
+
+The default is off. If you set LOG=1 or LOG_LEVEL or TRACE/DEBUG/VERBOSE/QUIET,
+then the default will be on. It defaults to off if you set LOG=0 or
+LOG_LEVEL=off.
+
+=head2 log_level => str (default: warning, or from env)
+
+Set default log level. The default can also be set via
+LOG_LEVEL/TRACE/DEBUG/VERBOSE/QUIET.
 
 =head1 METHODS
 
@@ -673,6 +689,19 @@ Set the default of C<table_column_orders> in C<format_options> in result
 metadata, similar to what's implemented in L<Perinci::Result::Format> and
 L<Data::Format::Pretty::Console>.
 
+=head1 RESULT METADATA
+
+All those supported by L<Perinci::CmdLine::Base>, plus:
+
+=head2 x.hint.result_binary => bool
+
+If set to true, then when formatting to C<text> formats, this class won't print
+any newline to keep the data being printed unmodified.
+
+=head1 SEE ALSO
+
+L<Perinci::CmdLine::Any>, L<Perinci::CmdLine::Classic>
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Perinci-CmdLine-Lite>.
@@ -689,21 +718,13 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
-=head1 SEE ALSO
-
-L<Perinci::CmdLine::Any>
-
-L<Perinci::CmdLine::Classic>
-
-L<Perinci::CmdLine::Inline>
-
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017, 2016, 2015, 2014 by perlancar@cpan.org.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
